--[[
    This is a stab at making a menu system.  I didn't like the one that
    blizzard had for drop downs.  It could be seriously
    rewritten so that it was more generally useful and easier to follow.

    Thanks to blizz for the good reference point though.
--]]

local ZooFu = ZooKeeper:NewClass('Frame')
ZooKeeper.ZooFuFrame = ZooFu

TYPE_MOUNTS = "MOUNT"
TYPE_COMPANIONS = "CRITTER"

--local references
local _G = getfenv(0)
local L = LibStub('AceLocale-3.0'):GetLocale('ZooKeeper')

--frame constructor
local lastID = 1
local buttonID = 1
local sideFrameID = 1
local buttonHeightPadding = 2
local buttonWidthPadding = 5
local iconWidth = 16
local minButtonWidth = 80
local textPadding = 5
local minButtonHeight = 15
local showTime = 1

function ZooFu:New(titleText, db)
    local f = self:Bind(CreateFrame('Frame', format('ZooFuFrame%d', lastID), UIParent, 'ZooFuMenuTemplate'))
    f.db = db
    f:CreateMenu()

    f:SetBackdropColor(0, 0, 0, 0.6)

    table.insert(UISpecialFrames, f:GetName())
    lastID = lastID + 1

    return f
end

function ZooFu:CreateMenu()
    self:CreateButtonData()
    self:CreateMenuButtons(UIParent, self, self.buttonList)

    self:SetScript("OnEnter", function(self) self:StopCounting(self) end);
    self:SetScript("OnLeave", function(self) self:StartCounting(self) end);
    self:SetScript("OnUpdate", function(self, elapsed) self:OnUpdate(self, elapsed) end);
end

function ZooFu:ShowMenu(fuBarFrame)
    self:SetPoint("TOPLEFT", fuBarFrame, "BOTTOMLEFT", 0, 2)
    if not self:IsShown() then
        self:Show()
    end
end

function ZooFu:OnClick(frame)
    local func = frame.func
    if func then
        func(unpack(frame.args));
        self:Hide()
        self:HideSides(self)
    end
end

function ZooFu:OnUpdate(self, elapsed)
    if ( not self.showTimer ) then
        return;
    elseif ( self.showTimer < 0 ) then
        self:Hide();
        self:HideSides(self);
        self.showTimer = nil;
    else
        self.showTimer = self.showTimer - elapsed;
    end
end

-- Start the countdown on a frame
function ZooFu:StartCounting(self)
    self.showTimer = showTime;
end

-- Stop the countdown on a frame
function ZooFu:StopCounting(self)
    self.showTimer = nil;
end

function ZooFu:HideSides(parent)
    local sidef
    for i = 1, getn(parent.sideFrames) do
        sidef = parent.sideFrames[i]
        sidef:Hide();
    end
end

function ZooFu:OnEnter(self, frame)
    self:StopCounting(self);

    if frame then
        self:HideSides(frame.parent);
        if frame.sideFrame then
            frame.sideFrame:Show();
        end
        getglobal(frame:GetName().."Highlight"):Show();
    end
end

function ZooFu:OnLeave(self, frame)
    self:StartCounting(self)

    if frame then
        getglobal(frame:GetName().."Highlight"):Hide();
    end
end

--[[
    Button logic
--]]

function ZooFu:CreateButtonData()
    self.buttonList = {}
    self.buttonList[1] = {
        name = L["CompanionMenuName"]
    }
    self.buttonList[2] = {
        name = L["MountMenuName"]
    }

    self.buttonList[1].list = self:FillInList(TYPE_COMPANIONS)
    self.buttonList[2].list = self:FillInList(TYPE_MOUNTS)
end

function ZooFu:FillInList(mode)
    local creatureID, creatureName, spellID, icon, active;
    local numPets = GetNumCompanions(mode)

    list = {}

    local names = {}
    local translator = {}
    for i = 1, numPets do
        creatureID, creatureName, spellID, icon, active = GetCompanionInfo(mode, i)
        table.insert(names, creatureName)
        translator[creatureName] = i
    end
    table.sort(names)

    for i = 1, numPets do
        creatureName = names[i]
        creatureID = translator[creatureName]
        list[i] = {}
        list[i].name = creatureName
        list[i].func = function(mode, creatureID) CallCompanion(mode, creatureID) end;
        list[i].args = {mode, creatureID}
    end

    return list
end

function ZooFu:CreateMenuButtons(parentFrame, currentFrame, currentList)
    local b, last, normalText
    local width, height
    local totalWidth = 1
    local totalHeight = 0

    currentFrame.sideFrames = {}

    for i = 1, getn(currentList) do
        b = currentList[i]
        b.relativeTo = last

        if not b.frame then
            self:CreateButton(b, currentFrame)
        end

        if b.list and getn(b.list) > 0 then
            self:CreateSideMenu(b)
            self:CreateMenuButtons(currentFrame, b.frame.sideFrame, b.list)
        end

        -- get height and width
        height = b.frame:GetTextHeight()
        width = b.frame:GetTextWidth()
        if width < minButtonWidth then
            width = minButtonWidth
        end
        if height < minButtonHeight then
            height = minButtonHeight
        end
        height = height + buttonHeightPadding
        width = width + (2 * buttonWidthPadding) + iconWidth

        -- adjust totals
        if width > totalWidth then
            totalWidth = width
        end
        totalHeight = totalHeight + height + buttonHeightPadding

        b.frame:SetWidth(width)
        b.frame:SetHeight(height)

        last = b
    end

    for i = 1, getn(currentList) do
        b = currentList[i]
        b.frame:SetWidth(totalWidth)
    end

    totalHeight = totalHeight + buttonHeightPadding;
    currentFrame:SetWidth(totalWidth);
    currentFrame:SetHeight(totalHeight);
end

function ZooFu:CreateSideMenu(b)
    b.frame.sideFrame = CreateFrame("Frame", format('ZooFuSideFrame%d', sideFrameID), b.frame.parent, 'ZooFuMenuTemplate');
    b.frame.sideFrame.top = self;
    b.frame.sideFrame:ClearAllPoints()
    b.frame.sideFrame:SetPoint("TOPLEFT", b.frame, "TOPRIGHT", -5, 0)
    b.frame.sideFrame:SetScript("OnEnter", function(self) self.top:OnEnter(self.top); end)
    b.frame.sideFrame:SetScript("OnLeave", function(self) self.top:OnLeave(self.top); end)
    b.frame.sideFrame:SetBackdropColor(0, 0, 0, 0.6)

    if getn(b.list) > 0 then
        local arrowFrame = getglobal(b.frame:GetName().."ExpandArrow")
        arrowFrame:Show();
    end

    b.frame.parent.sideFrames[sideFrameID] = b.frame.sideFrame;

    sideFrameID = sideFrameID + 1
end

function ZooFu:CreateButton(b, parent)
    b.buttonID = buttonID
    b.frame = CreateFrame("Button", format('ZooFuMenuButton%d', buttonID), parent, 'ZooFuMenuButtonTemplate');
    b.frame.top = self;
    b.frame.parent = parent
    b.frame.func = b.func
    b.frame.args = b.args

    if b.relativeTo then
        b.frame:SetPoint("TOPLEFT", b.relativeTo.frame, "BOTTOMLEFT", 0, (-1 * buttonHeightPadding))
    else
        b.frame:SetPoint("TOPLEFT", parent, "TOPLEFT", buttonWidthPadding, (-1 * buttonHeightPadding))
    end

    local normalText = getglobal(b.frame:GetName().."NormalText");
    normalText:ClearAllPoints()
    normalText:SetPoint("LEFT", b.frame, "LEFT", textPadding, 0);

    normalText:SetText("|cFFFFFFFF"..b.name.."|r")
    b.frame:SetScript("OnEnter", function(self) self.top:OnEnter(self.top, self) end)
    b.frame:SetScript("OnLeave", function(self) self.top:OnLeave(self.top, self) end)

    if b.frame.func then
        if not b.frame.args then
            b.frame.args = {}
        end
        b.frame:Enable();
        b.frame:SetScript("OnClick", function(self) self.top:OnClick(self) end);
    end

    local checkmark = getglobal(b.frame:GetName().."Check");
    if b.checkmark then
        checkmark:Show();
    else
        checkmark:Hide();
    end

    buttonID = buttonID + 1
end

